@@ -1,26 +1,19 @@
-"""Metods to add arguments based on class/method/function signatures."""
+"""Methods to add arguments based on class/method/function signatures."""
 
+import dataclasses
 import inspect
 import re
 from argparse import SUPPRESS
-from functools import wraps
 from typing import Any, Callable, List, Optional, Set, Tuple, Type, Union
 
-from .actions import _ActionConfigLoad, _ActionHelpClassPath
-from .namespace import Namespace
-from .typehints import ActionTypeHint, ClassType, is_optional, LazyInitBaseClass
+from .actions import _ActionConfigLoad
+from .optionals import get_doc_short_description
+from .parameter_resolvers import get_signature_parameters, ParamData
+from .typehints import ActionTypeHint, is_optional, LazyInitBaseClass
 from .typing import is_final_class
-from .util import get_import_path, _issubclass
-from .optionals import (
-    dataclasses_support,
-    docstring_parser_support,
-    import_dataclasses,
-    import_docstring_parse,
-)
-
+from .util import get_import_path, is_subclass, iter_to_set_str, LoggerProperty
 
 __all__ = [
-    'class_from_function',
     'compose_dataclasses',
     'SignatureArguments',
 ]
@@ -30,7 +23,7 @@ kinds = inspect._ParameterKind
 inspect_empty = inspect._empty
 
 
-class SignatureArguments:
+class SignatureArguments(LoggerProperty):
     """Methods to add arguments based on signatures to an ArgumentParser instance."""
 
     def add_class_arguments(
@@ -73,21 +66,17 @@ class SignatureArguments:
         if default and not (isinstance(default, LazyInitBaseClass) and isinstance(default, theclass)):
             raise ValueError(f'Expected "default" parameter to be a lazy instance of the class, got: {default}.')
 
-        skip_first = not _issubclass(theclass, ClassFromFunctionBase)
-
         added_args = self._add_signature_arguments(
-            inspect.getmro(theclass),
+            theclass,
+            None,
             nested_key,
             as_group,
             as_positional,
             skip,
             fail_untyped,
             sub_configs=sub_configs,
-            docs_func=get_class_init_and_base_docstrings,
-            sign_func=get_class_signature_functions,
             instantiate=instantiate,
             linked_targets=linked_targets,
-            skip_first=skip_first,
         )
 
         if default:
@@ -138,17 +127,16 @@ class SignatureArguments:
         if not hasattr(theclass, themethod) or not callable(getattr(theclass, themethod)):
             raise ValueError('Expected "themethod" argument to be a callable member of the class.')
 
-        skip_first = not isinstance(inspect.getattr_static(theclass, themethod), staticmethod)
-        themethods = [getattr(x, themethod) for x in inspect.getmro(theclass) if hasattr(x, themethod)]
-
-        return self._add_signature_arguments(themethods,
-                                             nested_key,
-                                             as_group,
-                                             as_positional,
-                                             skip,
-                                             fail_untyped,
-                                             sub_configs=sub_configs,
-                                             skip_first=skip_first)
+        return self._add_signature_arguments(
+            theclass,
+            themethod,
+            nested_key,
+            as_group,
+            as_positional,
+            skip,
+            fail_untyped,
+            sub_configs=sub_configs,
+        )
 
 
     def add_function_arguments(
@@ -184,43 +172,42 @@ class SignatureArguments:
         if not callable(function):
             raise ValueError('Expected "function" argument to be a callable object.')
 
-        return self._add_signature_arguments([function],
-                                             nested_key,
-                                             as_group,
-                                             as_positional,
-                                             skip,
-                                             fail_untyped,
-                                             sub_configs=sub_configs)
+        return self._add_signature_arguments(
+            function,
+            None,
+            nested_key,
+            as_group,
+            as_positional,
+            skip,
+            fail_untyped,
+            sub_configs=sub_configs,
+        )
 
 
     def _add_signature_arguments(
         self,
-        objects,
+        function_or_class,
+        method_name,
         nested_key: Optional[str],
-        as_group: bool,
-        as_positional: bool,
-        skip: Optional[Set[str]],
-        fail_untyped: bool,
+        as_group: bool = True,
+        as_positional: bool = False,
+        skip: Optional[Set[str]] = None,
+        fail_untyped: bool = True,
         sub_configs: bool = False,
-        docs_func: Callable = lambda x: [x.__doc__],
-        sign_func: Callable = lambda x: [(v, v) for v in x],  # type: ignore
-        skip_first: bool = False,
         instantiate: bool = True,
         linked_targets: Optional[Set[str]] = None,
     ) -> List[str]:
         """Adds arguments from parameters of objects based on signatures and docstrings.
 
         Args:
-            objects: Objects from which to add signatures.
+            function_or_class: Object from which to add arguments.
+            method_name: Class method from which to add arguments.
             nested_key: Key for nested namespace.
             as_group: Whether arguments should be added to a new argument group.
             as_positional: Whether to add required parameters as positional arguments.
             skip: Names of parameters that should be skipped.
             fail_untyped: Whether to raise exception if a required parameter does not have a type.
             sub_configs: Whether subclass type hints should be loadable from inner config file.
-            docs_func: Function that returns docstrings for a given object.
-            sign_func: Function that returns signature functions for a given object.
-            skip_first: Whether to skip first argument, i.e., skip self of class methods.
             instantiate: Whether the class group should be instantiated by :code:`instantiate_classes`.
 
         Returns:
@@ -229,53 +216,27 @@ class SignatureArguments:
         Raises:
             ValueError: When there are required parameters without at least one valid type.
         """
-
-        def update_has_args_kwargs(base, has_args=True, has_kwargs=True):
-            params = list(inspect.signature(base).parameters.values())
-            has_args &= any(p._kind == kinds.VAR_POSITIONAL for p in params)
-            has_kwargs &= any(p._kind == kinds.VAR_KEYWORD for p in params)
-            return has_args, has_kwargs
-
-        ## Determine propagation of arguments ##
-        signatures = sign_func(objects)
-        add_types = [(True, True)]
-        has_args, has_kwargs = update_has_args_kwargs(signatures[0][1])
-        for num in range(1, len(signatures)):
-            if not (has_args or has_kwargs):
-                signatures = signatures[:num]
-                break
-            add_types.append((has_args, has_kwargs))
-            has_args, has_kwargs = update_has_args_kwargs(signatures[num][1], has_args, has_kwargs)
-
-        ## Gather docstrings ##
-        doc_group, doc_params = self._gather_docstrings([s[0] for s in signatures], docs_func)
+        params = get_signature_parameters(function_or_class, method_name, logger=self.logger)
 
         ## Create group if requested ##
-        group = self._create_group_if_requested(objects[0], nested_key, as_group, doc_group, instantiate=instantiate)
+        doc_group = get_doc_short_description(function_or_class, method_name, logger=self.logger)
+        component = getattr(function_or_class, method_name) if method_name else function_or_class
+        group = self._create_group_if_requested(component, nested_key, as_group, doc_group, instantiate=instantiate)
 
-        ## Add objects arguments ##
+        ## Add parameter arguments ##
         added_args: List[str] = []
-        if skip is None:
-            skip = set()
-        for (obj, func), (add_args, add_kwargs) in zip(signatures, add_types):
-            for num, param in enumerate(inspect.signature(func).parameters.values()):
-                if skip_first and num == 0:
-                    continue
-                self._add_signature_parameter(
-                    group,
-                    nested_key,
-                    param,
-                    obj,
-                    doc_params,
-                    added_args,
-                    skip,
-                    fail_untyped=fail_untyped,
-                    sub_configs=sub_configs,
-                    linked_targets=linked_targets or set(),
-                    as_positional=as_positional,
-                    add_args=add_args,
-                    add_kwargs=add_kwargs,
-                )
+        for param in params:
+            self._add_signature_parameter(
+                group,
+                nested_key,
+                param,
+                added_args,
+                skip,
+                fail_untyped=fail_untyped,
+                sub_configs=sub_configs,
+                linked_targets=linked_targets,
+                as_positional=as_positional,
+            )
 
         return added_args
 
@@ -285,27 +246,25 @@ class SignatureArguments:
         group,
         nested_key: Optional[str],
         param,
-        obj: Any,
-        doc_params: dict,
         added_args: List[str],
-        skip: Set[str],
+        skip: Optional[Set[str]] = None,
         fail_untyped: bool = True,
         as_positional: bool = False,
         sub_configs: bool = False,
         instantiate: bool = True,
         linked_targets: Optional[Set[str]] = None,
-        add_args: bool = True,
-        add_kwargs: bool = True,
         default: Any = inspect_empty,
         **kwargs
     ):
         name = param.name
-        kind = param._kind
+        kind = param.kind
         annotation = param.annotation
         if default == inspect_empty:
             default = param.default
         is_required = default == inspect_empty
-        skip_message = f'Skipping parameter "{name}" from "{getattr(obj, "__name__", obj)}" because of: '
+        src = (param.parent.__name__+'.' if param.parent else '')
+        src += iter_to_set_str(x.__name__ for x in (param.component if isinstance(param.component, tuple) else [param.component]))
+        skip_message = f'Skipping parameter "{name}" from "{src}" because of: '
         if not fail_untyped and annotation == inspect_empty:
             annotation = Any
             default = None if is_required else default
@@ -317,21 +276,15 @@ class SignatureArguments:
            (not is_required and name[0] == '_') or \
            (annotation == inspect_empty and not is_required and default is None):
             return
-        elif is_required and not add_args:
-            self.logger.debug(skip_message+'Positional parameter but *args not propagated.')  # type: ignore
-            return
-        elif not is_required and not add_kwargs:
-            self.logger.debug(skip_message+'Keyword parameter but **kwargs not propagated.')  # type: ignore
-            return
-        elif name in skip:
-            self.logger.debug(skip_message+'Parameter requested to be skipped.')  # type: ignore
+        elif skip and name in skip:
+            self.logger.debug(skip_message+'Parameter requested to be skipped.')
             return
         if is_factory_class(default):
-            default = obj.__dataclass_fields__[name].default_factory()
+            default = param.parent.__dataclass_fields__[name].default_factory()
         if annotation == inspect_empty and not is_required:
             annotation = type(default)
         if 'help' not in kwargs:
-            kwargs['help'] = doc_params.get(name)
+            kwargs['help'] = param.doc
         if not is_required:
             kwargs['default'] = default
             if default is None and not is_optional(annotation, object):
@@ -339,40 +292,47 @@ class SignatureArguments:
         elif not as_positional:
             kwargs['required'] = True
         is_subclass_typehint = False
+        is_final_class_typehint = is_final_class(annotation)
+        dest = (nested_key+'.' if nested_key else '') + name
+        args = [dest if is_required and as_positional else '--'+dest]
         if annotation in {str, int, float, bool} or \
-           _issubclass(annotation, (str, int, float)) or \
-           is_final_class(annotation) or \
+           is_subclass(annotation, (str, int, float)) or \
+           is_final_class_typehint or \
            is_pure_dataclass(annotation):
             kwargs['type'] = annotation
         elif annotation != inspect_empty:
             try:
                 is_subclass_typehint = ActionTypeHint.is_subclass_typehint(annotation)
-                enable_path = is_subclass_typehint and sub_configs
-                kwargs['action'] = ActionTypeHint(typehint=annotation, enable_path=enable_path)
-            except ValueError as ex:
-                self.logger.debug(skip_message+str(ex))  # type: ignore
-        if 'type' in kwargs or 'action' in kwargs:
-            dest = (nested_key+'.' if nested_key else '') + name
-            if dest in added_args:
-                self.logger.debug(skip_message+'Argument already added.')  # type: ignore
-            else:
-                opt_str = dest if is_required and as_positional else '--'+dest
+                kwargs['type'] = annotation
+                sub_add_kwargs = None
                 if is_subclass_typehint:
-                    help_action = group.add_argument(f'--{dest}.help', action=_ActionHelpClassPath(baseclass=annotation))
                     prefix = name + '.init_args.'
-                    subclass_skip = {s[len(prefix):] for s in skip if s.startswith(prefix)}
-                    help_action.sub_add_kwargs = {'fail_untyped': fail_untyped, 'skip': subclass_skip}
-                action = group.add_argument(opt_str, **kwargs)
-                action.sub_add_kwargs = {
-                    'fail_untyped': fail_untyped,
-                    'sub_configs': sub_configs,
-                    'instantiate': instantiate,
-                }
-                if is_subclass_typehint and len(subclass_skip) > 0:
-                    action.sub_add_kwargs['skip'] = subclass_skip
-                added_args.append(dest)
+                    subclass_skip = {s[len(prefix):] for s in skip or [] if s.startswith(prefix)}
+                    sub_add_kwargs = {'fail_untyped': fail_untyped, 'skip': subclass_skip}
+                args = ActionTypeHint.prepare_add_argument(
+                    args=args,
+                    kwargs=kwargs,
+                    enable_path=is_subclass_typehint and sub_configs,
+                    container=group,
+                    sub_add_kwargs=sub_add_kwargs,
+                )
+            except ValueError as ex:
+                self.logger.debug(skip_message+str(ex))
+        if 'type' in kwargs or 'action' in kwargs:
+            sub_add_kwargs = {
+                'fail_untyped': fail_untyped,
+                'sub_configs': sub_configs,
+                'instantiate': instantiate,
+            }
+            if is_final_class_typehint:
+                kwargs.update(sub_add_kwargs)
+            action = group.add_argument(*args, **kwargs)
+            action.sub_add_kwargs = sub_add_kwargs
+            if is_subclass_typehint and len(subclass_skip) > 0:
+                action.sub_add_kwargs['skip'] = subclass_skip
+            added_args.append(dest)
         elif is_required and fail_untyped:
-            raise ValueError(f'Required parameter without a type for {obj} parameter "{name}".')
+            raise ValueError(f'Required parameter without a type for "{src}" parameter "{name}".')
 
 
     def add_dataclass_arguments(
@@ -398,14 +358,13 @@ class SignatureArguments:
             ValueError: When not given a dataclass.
             ValueError: When default is not instance of or kwargs for theclass.
         """
-        dataclasses = import_dataclasses('add_dataclass_arguments')
         if not is_pure_dataclass(theclass):
             raise ValueError(f'Expected "theclass" argument to be a pure dataclass, given {theclass}')
 
-        doc_group, doc_params = self._gather_docstrings([theclass], get_class_init_and_base_docstrings)
+        doc_group = get_doc_short_description(theclass, logger=self.logger)
         for key in ['help', 'title']:
             if key in kwargs and kwargs[key] is not None:
-                doc_group = strip_title(kwargs.pop(key))
+                doc_group = kwargs.pop(key)
         group = self._create_group_if_requested(theclass, nested_key, as_group, doc_group, config_load_type=theclass)
 
         defaults = {}
@@ -420,17 +379,13 @@ class SignatureArguments:
             defaults = dataclasses.asdict(default)
 
         added_args: List[str] = []
-        skip: Set[str] = set()
-        params = inspect.signature(theclass.__init__).parameters
+        params = {p.name: p for p in get_signature_parameters(theclass, None, logger=self.logger)}
         for field in dataclasses.fields(theclass):
             self._add_signature_parameter(
                 group,
                 nested_key,
                 params[field.name],
-                theclass,
-                doc_params,
                 added_args,
-                skip,
                 default=defaults.get(field.name, inspect_empty),
             )
 
@@ -445,8 +400,8 @@ class SignatureArguments:
         skip: Optional[Set[str]] = None,
         instantiate: bool = True,
         required: bool = False,
-        metavar: str = '{"class_path":...[,"init_args":...]}',
-        help: str = 'Dictionary with "class_path" and "init_args" for any subclass of %(baseclass_name)s.',
+        metavar: str = 'CONFIG | CLASS_PATH_OR_NAME | .INIT_ARG_NAME VALUE',
+        help: str = 'One or more arguments specifying "class_path" and "init_args" for any subclass of %(baseclass_name)s.',
         **kwargs
     ):
         """Adds arguments to allow specifying any subclass of the given base class.
@@ -478,7 +433,7 @@ class SignatureArguments:
         if not all(inspect.isclass(c) for c in baseclass):
             raise ValueError('Expected "baseclass" argument to be a class or a tuple of classes.')
 
-        doc_group = self._gather_docstrings(baseclass, get_class_init_and_base_docstrings)[0]
+        doc_group = get_doc_short_description(baseclass[0], logger=self.logger)
         group = self._create_group_if_requested(
             baseclass,
             nested_key,
@@ -490,12 +445,10 @@ class SignatureArguments:
         )
 
         added_args: List[str] = []
-        if skip is None:
-            skip = set()
-        else:
-            skip = set(nested_key+'.init_args.'+s for s in skip)
-        param = Namespace(name=nested_key, _kind=None, annotation=Union[baseclass])
-        str_baseclass = '{' + ', '.join(get_import_path(x) for x in baseclass) + '}'
+        if skip is not None:
+            skip = {nested_key+'.init_args.'+s for s in skip}
+        param = ParamData(name=nested_key, annotation=Union[baseclass], component=baseclass)
+        str_baseclass = iter_to_set_str(get_import_path(x) for x in baseclass)
         kwargs.update({
             'metavar': metavar,
             'help': (help % {'baseclass_name': str_baseclass}),
@@ -506,8 +459,6 @@ class SignatureArguments:
             group,
             None,
             param,
-            Union[baseclass],
-            {},
             added_args,
             skip,
             sub_configs=True,
@@ -516,26 +467,6 @@ class SignatureArguments:
         )
 
 
-    def _gather_docstrings(self, objects, docs_func):
-        doc_group = None
-        doc_params = {}
-        if docstring_parser_support:
-            docstring_parse, DocstringParseError = import_docstring_parse('_gather_docstrings', True)
-            for base in objects:
-                for doc in docs_func(base):
-                    try:
-                        docstring = docstring_parse(doc)
-                    except (ValueError, DocstringParseError) as ex:
-                        self.logger.debug(f'Failed parsing docstring for {base}: {ex}')
-                    else:
-                        if docstring.short_description and not doc_group:
-                            doc_group = docstring.short_description
-                        for param in docstring.params:
-                            if param.arg_name not in doc_params:
-                                doc_params[param.arg_name] = param.description
-        return strip_title(doc_group), doc_params
-
-
     def _create_group_if_requested(self, obj, nested_key, as_group, doc_group, config_load=True, config_load_type=None, required=False, instantiate=True):
         if required:
             if nested_key is None:
@@ -547,7 +478,7 @@ class SignatureArguments:
             if doc_group is None:
                 doc_group = str(obj)
             name = obj.__name__ if nested_key is None else nested_key
-            group = self.add_argument_group(doc_group, name=name)
+            group = self.add_argument_group(strip_title(doc_group), name=name)
             if config_load and nested_key is not None:
                 group.add_argument('--'+nested_key, action=_ActionConfigLoad(basetype=config_load_type))
             if inspect.isclass(obj) and nested_key is not None and instantiate:
@@ -568,44 +499,24 @@ def group_instantiate_class(group, cfg):
     parent[key] = group.group_class(**value)
 
 
-def get_class_init_and_base_docstrings(value):
-    return [value.__init__.__doc__, value.__doc__]
-
-
-def get_class_signature_functions(classes):
-    signatures = []
-    for num, cls in enumerate(classes):
-        if cls.__new__ is not object.__new__ and not any(cls.__new__ is c.__new__ for c in classes[num+1:]):
-            signatures.append((cls, cls.__new__))
-        if not any(cls.__init__ is c.__init__ for c in classes[num+1:]):
-            signatures.append((cls, cls.__init__))
-    return signatures
-
-
 def strip_title(value):
     if value is not None:
         return re.sub(r'\.$', '', value.strip())
 
 
 def is_factory_class(value):
-    result = False
-    if dataclasses_support:
-        dataclasses = import_dataclasses('is_default_factory_class')
-        result = value.__class__ == dataclasses._HAS_DEFAULT_FACTORY_CLASS
-    return result
+    return value.__class__ == dataclasses._HAS_DEFAULT_FACTORY_CLASS
 
 
 def is_pure_dataclass(value):
-    if not dataclasses_support or not inspect.isclass(value):
+    if not inspect.isclass(value):
         return False
-    dataclasses = import_dataclasses('is_pure_dataclass')
     classes = [c for c in inspect.getmro(value) if c != object]
     return all(dataclasses.is_dataclass(c) for c in classes)
 
 
 def compose_dataclasses(*args):
     """Returns a pure dataclass inheriting all given dataclasses and properly handling __post_init__."""
-    dataclasses = import_dataclasses('compose_dataclasses')
 
     @dataclasses.dataclass
     class ComposedDataclass(*args):
@@ -615,33 +526,3 @@ def compose_dataclasses(*args):
                     arg.__post_init__(self)
 
     return ComposedDataclass
-
-
-class ClassFromFunctionBase:
-    pass
-
-
-def class_from_function(func: Callable[..., ClassType]) -> Type[ClassType]:
-    """Creates a dynamic class which if instantiated is equivalent to calling func.
-
-    Args:
-        func: A function that returns an instance of a class. It must have a return type annotation.
-    """
-    func_return = inspect.signature(func).return_annotation
-    if isinstance(func_return, str):
-        caller_frame = inspect.currentframe().f_back  # type: ignore
-        func_return = caller_frame.f_locals.get(func_return) or caller_frame.f_globals.get(func_return)  # type: ignore
-        if func_return is None:
-            raise ValueError(f'Unable to dereference {func_return} the return type of {func}.')
-
-    @wraps(func)
-    def __new__(cls, *args, **kwargs):
-        return func(*args, **kwargs)
-
-    class ClassFromFunction(func_return, ClassFromFunctionBase):  # type: ignore
-        pass
-
-    ClassFromFunction.__new__ = __new__  # type: ignore
-    ClassFromFunction.__doc__ = func.__doc__
-    ClassFromFunction.__name__ = func.__name__
-    return ClassFromFunction
