@@ -349,8 +349,9 @@ class ActionTypeHint(Action):
         for num, val in enumerate(value):
             try:
                 orig_val = val
+                enable_path = self._enable_path and not isinstance(val, NestedArg)
                 try:
-                    val, config_path = parse_value_or_config(val, enable_path=self._enable_path)
+                    val, config_path = parse_value_or_config(val, enable_path=enable_path)
                 except get_loader_exceptions():
                     config_path = None
                 path_meta = val.pop('__path__', None) if isinstance(val, dict) else None
@@ -413,7 +414,10 @@ class ActionTypeHint(Action):
 
     def extra_help(self):
         extra = ''
-        if self.is_subclass_typehint(self, all_subtypes=False):
+        if (
+            self.is_subclass_typehint(self, all_subtypes=False) or
+            get_typehint_origin(self._typehint) in {Type, type}
+        ):
             class_paths = get_all_subclass_paths(self._typehint)
             if class_paths:
                 extra = ', known subclasses: '+', '.join(class_paths)
@@ -713,7 +717,7 @@ def get_all_subclass_paths(cls: Type) -> List[str]:
         for subclass in cl.__subclasses__() if hasattr(cl, '__subclasses__') else []:
             add_subclasses(subclass)
 
-    if get_typehint_origin(cls) == Union:
+    if get_typehint_origin(cls) in {Union, Type, type}:
         for arg in cls.__args__:
             if ActionTypeHint.is_subclass_typehint(arg) and arg not in {object, type}:
                 add_subclasses(arg)
