@@ -1,12 +1,12 @@
 """Collection of types and type generators."""
 
 import operator
+import os
 import re
 import uuid
 from datetime import timedelta
 from typing import Any, Callable, Dict, List, Optional, Pattern, Tuple, Type, Union
-from .namespace import Namespace
-from .optionals import typing_extensions_import
+from .optionals import final
 from .util import Path
 
 
@@ -301,14 +301,6 @@ def add_type(type_class: Type, uniqueness_key: Optional[Tuple], type_check: Call
     register_type(type_class, type_class._type, **kwargs)  # type: ignore
 
 
-final = typing_extensions_import('final')
-if not getattr(final(Namespace()), '__final__', False):
-    def final(cls):  # pylint: disable=function-redefined
-        """Decorator to make a class `final`, i.e. it shouldn't be subclassed."""
-        setattr(cls, '__final__', True)
-        return cls
-
-
 def is_final_class(cls):
     """Checks whether a class is final, i.e. decorated with ``final``."""
     return getattr(cls, '__final__', False)
@@ -338,6 +330,7 @@ Path_dw = path_type('dw', docstring='str pointing to a directory that exists and
 Path_dc = path_type('dc', docstring='str pointing to a directory that can be created if it does not exist')
 Path_drw = path_type('drw', docstring='str pointing to a directory that exists and is readable and writeable')
 
+register_type(os.PathLike, str, str)
 register_type(complex)
 register_type(uuid.UUID)
 
