@@ -803,6 +803,22 @@ class TypeHintsTests(unittest.TestCase):
         self.assertRaises(ParserError, lambda: parser.parse_args([f'--config={config}']))
 
 
+    def test_help_known_subclasses_class(self):
+        parser = ArgumentParser()
+        parser.add_argument('--cal', type=Calendar)
+        out = StringIO()
+        parser.print_help(out)
+        self.assertIn('known subclasses: calendar.Calendar,', out.getvalue())
+
+
+    def test_help_known_subclasses_type(self):
+        parser = ArgumentParser()
+        parser.add_argument('--cal', type=Type[Calendar])
+        out = StringIO()
+        parser.print_help(out)
+        self.assertIn('known subclasses: calendar.Calendar,', out.getvalue())
+
+
     def test_typehint_serialize_list(self):
         parser = ArgumentParser()
         action = parser.add_argument('--list', type=Union[PositiveInt, List[PositiveInt]])
@@ -980,6 +996,23 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         self.assertIn('(type: Path_drw_skip_check, default: test)', out.getvalue())
 
 
+    def test_path_like_within_subclass(self):
+        class Data:
+            def __init__(self, path: Optional[os.PathLike] = None):
+                pass
+
+        data_path = pathlib.Path('data.json')
+        data_path.write_text('{"a": 1}')
+
+        parser = ArgumentParser()
+        parser.add_argument('--data', type=Data, enable_path=True)
+
+        with mock_module(Data) as module:
+            cfg = parser.parse_args([f'--data={module}.Data', f'--data.path={data_path}'])
+            self.assertEqual(cfg.data.class_path, f'{module}.Data')
+            self.assertEqual(cfg.data.init_args, Namespace(path=str(data_path)))
+
+
     def test_list_append_default_config_files(self):
         config_path = pathlib.Path(self.tmpdir, 'config.yaml')
         parser = ArgumentParser(default_config_files=[str(config_path)])
@@ -1009,6 +1042,18 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             self.assertEqual(cfg.nums, [0, 1, 2])
 
 
+    def test_list_append_default_config_files_subcommand(self):
+        config_path = pathlib.Path(self.tmpdir, 'config.yaml')
+        parser = ArgumentParser(default_config_files=[str(config_path)])
+        subcommands = parser.add_subcommands()
+        subparser = ArgumentParser()
+        subparser.add_argument('--nums', type=List[int], default=[0])
+        subcommands.add_subcommand('sub', subparser)
+        config_path.write_text('sub:\n  nums: [1]\n')
+        cfg = parser.parse_args(['sub', '--nums+=2'])
+        self.assertEqual(cfg.sub.nums, [1, 2])
+
+
     def test_class_type_with_default_config_files(self):
         config = {
             'class_path': 'calendar.Calendar',
